import unittest

from .api import TestDesigner, TestConnection
from .utils import TestUtils

__all__ = [
    'TestDesigner',
    'TestConnection',
    'TestUtils',
]

if __name__ == '__main__':
    unittest.main()



import logging
import sys
import os.path as path
import os
import subprocess

logger = logging.getLogger(__file__)


def windows_platform() -> bool:
    return 'win' in sys.platform


def get_platform_path(version: str = '') -> str:
    """
    Вычисляет путь к исполняемому файлу платформы

    :param version: Версия платформы.
    :return:
    """

    if windows_platform():
        platform_path = __get_platform_path_windows(version)
    else:
        platform_path = __get_platform_path_linux(version)

    return platform_path


def __get_version_path(dir_1c, version) -> str:
    bin_path = 'bin\\1cv8.exe'
    exept_dir = ['common', 'conf']

    versions = [dir_name for dir_name in os.listdir(dir_1c) if dir_name not in exept_dir]

    if len(versions) == 0:
        return ''

    if version == '':
        # Получим максимальную версию
        return path.join(dir_1c, sorted(versions, key=get_version_weight, reverse=True)[0], bin_path)
    elif version in versions:
        return path.join(dir_1c, version, bin_path)
    else:
        return ''


def get_version_weight(element: str) -> int:
    """
    Вычисляет вес версии, для сравнения версий
    Подразумевается, что версии, для которых будет вычисляться октавы имеют одинаковые длины октавово
    и их длина не больше 4

    :param element:
    :return:
    """
    octs_first = element.split('.')
    octs_first.reverse()
    koef = 1000
    summ = 0
    for i, val in enumerate(octs_first):
        if len(val) > 4:
            logger.error(f'Длина октава {val} в версии {element} больше 4, вычисление веса не поддерживается.')
            raise ValueError('Ошибка вычисления веса версии, длинна октава не должна быть больше 4.')
        summ += int(val) * koef ** i
    return summ


def __get_platform_path_windows(version: str) -> str:
    version_path = __get_version_path(path.join(os.getenv('ProgramW6432'), '1cv8'), version)

    if version_path == '':
        version_path = __get_version_path(path.join(os.getenv('ProgramFiles'), '1cv8'), version)

    if version_path == '':
        if version == '':
            logger.critical('Не обнаружена установленная 1с. Выполнение невозможно.')
            raise EnvironmentError('Не обнаружена установленная 1с.')
        else:
            logger.critical(f'Не обнаружена установленная версия 1с номер версии:{version}. Выполнение невозможно.')
            raise EnvironmentError(f'Не обнаружена версия {version} 1с.')

    return version_path


def __get_platform_path_linux(version: str) -> str:
    platform_path = ''

    return platform_path


def execute_command(command: str, params: list, timeout: int = None) -> tuple:
    """
    Выполняет команду в системе.

    :param command: Команда
    :param params: Параметры команды
    :param timeout: Лимит времени на выполнение команды, после выхода за пределы будет возбуждено исключение.
    :return:
    """
    if windows_platform():
        result = __execute_windows_command(command, params, timeout)
    else:
        result = __execute_linux_command(command, params, timeout)
    return result


def __execute_windows_command(command: str, params: list, timeout: int) -> tuple:
    """
    Выполняет команду системы в windows

    :param command:
    :return:
    """
    try:
        process = subprocess.run(
            args=[command] + params,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            check=True,
            encoding=encoding(),
            timeout=timeout
        )
        if process.returncode == 0:
            msg = process.stdout
        else:
            msg = process.stderr
        return process.returncode, msg.strip()
    except subprocess.TimeoutExpired:
        return 1, 'Выполнение процесса вышло за рамки отведенного времени.'


def encoding() -> str:
    """
    Определяет кодировку в контексте системы.
        Спасибо (@vbondarevsky)
    :return:
    """
    if windows_platform():
        import ctypes
        return f"cp{ctypes.windll.kernel32.GetOEMCP()}"
    else:
        return (sys.stdout.encoding if sys.stdout.isatty() else
                sys.stderr.encoding if sys.stderr.isatty() else
                sys.getfilesystemencoding() or "utf-8")


def __execute_linux_command(command: str, params: list, timeout: int) -> str:
    """
    Выполняет команду системы в linux

    :param command:
    :return:
    """
    raise NotImplementedError('Не реализовано!')
    pass



from designer_cmd.utils import utils

import unittest
from unittest import mock


class TestUtils(unittest.TestCase):

    def setUp(self):
        pass

    def test_get_version_weight(self):
        weight = utils.get_version_weight('8.3.14.1232')
        self.assertEqual(weight, 8003015232)

    def test_get_version_weight_eseption(self):
        with self.assertRaises(ValueError) as error:
            utils.get_version_weight('8.3.14.123212')
        self.assertEqual(
            error.exception.args[0],
            'Ошибка вычисления веса версии, длинна октава не должна быть больше 4.',
            'Проверка на ошибочную версию не прошла.'
        )

    @mock.patch('os.listdir')
    def test_get_platform_path_exeption(self, os_listdir):

        check_version = '8.3.14.1232'
        os_listdir.return_value = [
            '8.3.11.1232',
            '8.3.12.1132',
            '8.3.14.1231',
            '8.2.14.1232',
            '8.3.15.1232',
        ]

        with self.assertRaises(EnvironmentError) as error:
            utils.get_platform_path(check_version)
        self.assertEqual(
            error.exception.args[0],
            f'Не обнаружена версия {check_version} 1с.',
            'Проверка на ошибку поиска не пройдена'
        )

    @mock.patch('os.listdir')
    def test_get_platform_path(self, os_listdir):
        pref_path = 'PATH'
        max_version = '8.3.17.1212'
        os_listdir.return_value = [
            '8.3.11.1232',
            '8.3.12.1132',
            '8.3.14.1231',
            '8.2.14.1232',
            '8.3.15.1232',
            max_version
        ]
        check_version = '8.2.14.1232'
        with mock.patch('os.getenv') as os_getenv:
            os_getenv.return_value = pref_path
            platform_path = utils.get_platform_path(check_version)
            self.assertEqual(
                platform_path,
                f'{pref_path}\\1cv8\\{check_version}',
                'Не прошла проверка не получение версии'
            )

            platform_path = utils.get_platform_path('')
            self.assertEqual(
                platform_path,
                f'{pref_path}\\1cv8\\{max_version}',
                'Не прошла проверка на получение последней версии'
            )

    def test_execute_command(self):
        result = utils.execute_command('dir', [])
        self.assertEqual(result[0], 0, 'Не удалось выполнить команду систему!')

        result = utils.execute_command('dir111', [])
        self.assertNotEqual(result[0], 0, 'Проверка на ошибочную команду провалилась!')

    def test_timout_exception(self):
        result = utils.execute_command('cmd.exe', ['/c', 'pause 10'], 1)
        self.assertTrue(result[0] == 1, 'Ошибка по таймауту не произошла')

    def tearDown(self):
        pass



from designer_cmd.api import Connection, Designer
import unittest
import os.path as path
import os
import shutil


class TestConnection(unittest.TestCase):

    def setUp(self):
        pass

    def test_init_base_path(self):
        with self.assertRaises(AttributeError) as cm:
            con = Connection(user='user')
        the_exception = cm.exception
        self.assertEqual(
            the_exception.args[0],
            'Для соедеинения не определен путь к базе!',
            'Полученно неожиданное исключение'
        )

    def test_get_connection_params_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_params(),
            [f'/F {path.abspath(loc_path)}', '/N user', '/P pass'],
            'Не верно определена строка подключени для файловой базы'
        )

    def test_get_connection_params_server(self):

        con = Connection(user='user', password='pass', server_path='192.168.1.1', server_base_ref='test')
        self.assertEqual(
            con.get_connection_params(),
            ['/S 192.168.1.1\\test', '/N user', '/P pass'],
            'Не верно определена строка подключени для серверной базы'
        )

    def test_get_connection_string_file(self):

        loc_path = 'path'

        con = Connection(user='user', password='pass', file_path=loc_path)
        self.assertEqual(
            con.get_connection_string(),
            f'Usr=user Pwd=pass File={path.abspath(loc_path)}',
            'Не верно определена строка подключени для файловой базы'
        )


def clear_folder(dir_path):
    if path.exists(dir_path):
        filelist = [f for f in os.listdir(dir_path)]
        for file_name in filelist:
            if '.gitkeep' in file_name:
                continue
            file_path = path.join(dir_path, file_name)
            if path.isdir(file_path):
                shutil.rmtree(file_path)
            else:
                os.remove(file_path)


class TestDesigner(unittest.TestCase):

    def setUp(self):
        test_data_dir = path.join(path.dirname(__file__), 'test_data')

        self.temp_path = path.join(test_data_dir, 'temp')
        self.test_base_path = path.join(test_data_dir, 'base')
        self.cf_path = path.join(test_data_dir, '1Cv8.cf')
        self.dt_path = path.join(test_data_dir, '1Cv8.dt')

        self.conn = self.db_connection()
        self.designer = Designer('', self.conn)

        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)

    def db_connection(self):
        return Connection(file_path=self.test_base_path)

    def prepare_base(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.updete_db_config()

    def test_load_create_db(self):
        self.designer.create_base()

    def test_load_conf_from_file(self):
        self.prepare_base()

    def test_update_cfg(self):
        self.designer.create_base()
        self.designer.load_config_from_file(self.cf_path)
        self.designer.updete_db_config()

    def test_dump_conf_to_file(self):
        self.prepare_base()
        cf_file_path = path.join(self.temp_path, '1Cv81.cf')
        self.designer.dump_config_to_file(cf_file_path)

        self.assertTrue(os.path.exists(cf_file_path), 'Выгрузка кофигурации не создана!')

    def test_dump_config_to_files(self):
        self.prepare_base()
        dir_xml_config_path = path.join(self.temp_path, 'xml_config')
        if not path.exists(dir_xml_config_path):
            os.mkdir(dir_xml_config_path)
        self.designer.dump_config_to_files(dir_xml_config_path)

        self.assertTrue(
            os.path.exists(os.path.join(dir_xml_config_path, 'Catalogs\\Справочник1.xml')),
            'Не обнаружена выгрузка в xml'
        )

    def test_compare_conf(self):
        self.prepare_base()
        report_path = os.path.join(self.temp_path, 'report.txt')
        self.designer.compare_config_with_file(self.cf_path, report_path)

        self.assertTrue(os.path.exists(report_path), 'Отчет о сравнении не создан!')

    def test_dump_db_to_file(self):
        self.prepare_base()
        new_dt_path = os.path.join(self.temp_path, '1c.dt')
        self.designer.dump_db_to_file(new_dt_path)

        self.assertTrue(os.path.exists(new_dt_path), 'Файл выгрузки не создан!')

    def test_load_db_from_file(self):
        self.designer.create_base()
        self.designer.load_db_from_file(self.dt_path)

    def tearDown(self):
        clear_folder(self.test_base_path)
        clear_folder(self.temp_path)



from designer_cmd.utils import get_platform_path, execute_command
import os
import logging
import tempfile

logger = logging.getLogger(__file__)


class Connection:
    """
    Описывает соединение с базой
    """
    def __init__(self, user: str= '',
                 password: str = '',
                 file_path: str = '',
                 server_path: str = '',
                 server_base_ref: str = '',
                 ib_name: str = '',
                 time_out: int = 3600):

        if file_path == '' and (server_path == '' or server_base_ref == '') and ib_name == '':
            raise AttributeError('Для соедеинения не определен путь к базе!')

        self.user = user
        self.password = password

        self.ib_name = ib_name

        self.file_path = os.path.abspath(file_path) if file_path != '' else ''
        self.server_path = server_path
        self.server_base_ref = server_base_ref

        self.__time_out = time_out

    @property
    def timeout(self):
        return self.__time_out

    @timeout.setter
    def timeout(self, value):
        self.__time_out = value

    def get_connection_params(self) -> list:
        """
        Возвращает строковое представление параметров для подключения по текущему описанию соединения

        :return: str : Текстовые параметры для командной строки
        """
        params = []
        if self.file_path != '':
            params += [f'/F {self.file_path}']
        elif self.ib_name != '':
            params += [f'/IBName {self.ib_name}']
        else:
            params += [f'/S {self.server_path}\\{self.server_base_ref}']

        if self.user != '':
            params.append(f'/N {self.user}')
            params.append(f'/P {self.password}')

        return params

    def get_connection_string(self):
        if self.ib_name != '':
            raise ValueError('Не возможно сформировать строку подключения для базы из списка.')

        if self.file_path != '':
            if self.user == '':
                return f'File={self.file_path}'
            else:
                return f'Usr={self.user} Pwd={self.password} File={self.file_path}'
        else:
            raise NotImplementedError('Создание базы в серверном варианте не реализованно.')

    def __repr__(self):
        if self.file_path != '':
            connection_path = f'File: {self.file_path}'
        elif self.ib_name != '':
            connection_path = f'IBName: {self.ib_name}'
        else:
            connection_path = f'Server: {self.server_path} base_ref: {self.server_base_ref}'
        connection_path += f'user: {self.user}' if self.user != '' else ''
        return connection_path


class Designer:

    def __init__(self, platform_version: str, connection: Connection):

        self.platform_version = platform_version
        self.connection = connection
        self.platform_path = get_platform_path(self.platform_version)

    def __execute_command(self, mode: str, command_params: list, connection_params_required: bool = True):
        params = [mode]
        params += command_params
        if connection_params_required:
            params += self.connection.get_connection_params()
        params += ['/DisableStartupDialogs /DisableStartupMessages']
        debug_file_name = self.add_debug_params(params)

        str_command = ' '.join(params)
        logger.debug(f'Выполняю команду {self.platform_path} {str_command}')

        result = execute_command(self.platform_path, params, self.connection.timeout)

        if result[0] != 0:
            logger.error(f'При выполнении команды произошла ошибка: {open(debug_file_name).read()} {result[1]}')
            os.remove(debug_file_name)
            raise SyntaxError('Не удалось выполнить команду!')
        os.remove(debug_file_name)

    def add_debug_params(self, params) -> str:
        debug_file_name = tempfile.mkstemp('.log')
        params.append('/Out')
        params.append(debug_file_name[1])
        os.close(debug_file_name[0])
        return debug_file_name[1]

    def create_base(self):
        """
        Создает базу данных.
        Соответствует режиму CREATEINFOBASE <строка соединения>

        """
        logger.info(f'Создаю базу по соединению: {self.connection}')
        params = [f'{self.connection.get_connection_string()}']
        self.__execute_command('CREATEINFOBASE', params, False)

    def updete_db_config(self, dynamic: bool = False, warnings_as_errors: bool = False):
        """
        Обновляет конфигурацию db. (соответствует команде /UpdateDBCfg)

        :return:
        """
        logger.info(f'Обновляю конфигурацию БД по соединению {self.connection}')
        params = [f'/UpdateDBCfg']
        self.__execute_command(f'DESIGNER', params)

    def load_db_from_file(self, file_path: str) -> None:
        """
        Загрузка базы из файла dt (соответствует команде /RestoreIB)

        :param file_path: str - Путь к файлу базы (dt.)
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(f'Загружаю файл dt {full_file_path} в БД по соединению {self.connection}')
        params = ['/RestoreIB', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_db_to_file(self, file_path: str) -> None:
        """
        Выгрузка базы в файл (соответствует команде /DumpIB)

        :param file_path: str - Путь к файлу для выгрзки базы (dt.)
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(f'Выгружаю файл dt по пути {full_file_path} из БД по соединению {self.connection}')
        params = ['/DumpIB', f'{full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def load_config_from_files(self, catalog_path: str) -> None:
        """
        Загружает конфигурацию из файлов (соответствует команде /LoadConfigFromFiles)

        :param catalog_path: str - путь к каталогу из которого необходимо произвести загрузку.
        :return:
        """
        full_catalog_path = os.path.abspath(catalog_path)
        logger.info(f'Загружаю конфигурацию из файлов {full_catalog_path} конфигурацию БД по соединению {self.connection}')
        params = [f'/LoadConfigFromFiles {full_catalog_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_config_to_files(self, catalog_path: str) -> None:
        """
        Выгружает конфигурацию в файлы (соответствует команде /DumpConfigToFiles)

        :param catalog_path: Каталог в который будет выгружен файл
        :return:
        """
        full_catalog_path = os.path.abspath(catalog_path)
        logger.info(
            f'Выгружаю конфигурацию в файлы {full_catalog_path} по соединению {self.connection}')
        params = [f'/DumpConfigToFiles {full_catalog_path}']
        self.__execute_command(f'DESIGNER', params)

    def load_config_from_file(self, file_path: str) -> None:
        """
        Загружает конфигурацию в базу из файла cf (соответствует команде /LoadCfg)

        :param file_path:
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(
            f'Загружаю конфигурацию из файла {full_file_path} в конфигурацию БД по соединению {self.connection}')
        params = [f'/LoadCfg {full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def dump_config_to_file(self, file_path: str) -> None:
        """
        Выполнить сохранение конфигурации в файл cf (соответствует команде /DumpCfg)
        :param file_path:
        :return:
        """
        full_file_path = os.path.abspath(file_path)
        logger.info(
            f'Сохраняю конфигурацию в файл {full_file_path} из конфигурации БД по соединению {self.connection}')
        params = [f'/DumpCfg {full_file_path}']
        self.__execute_command(f'DESIGNER', params)

    def merge_config_with_file(self, cf_file_path: str, settings_path: str) -> None:
        """
        Выполнить объединение текущей конфигурации с файлом (с использованием файла настроек).
        (соответствует команде /MergeCfg)

        :param cf_file_path: str : Путь к файлу cf для объединения с текущей конфой
        :param settings_path: str : Путь к файлу настроек объединения
        :return:
        """
        raise NotImplementedError

    def compare_config_with_file(self, cf_file_path: str, report_path: str) -> None:
        """
        Выполнить сравнение двух конфигураций и сформировать файл с отчетом о сравнении.
        (соответствует команде /CompareCfg)
        :param cf_file_path: Путь к файлу cf, который необходимо сравнить с конфигурацией.
        :param: report_path: Путь к файлу, в который необходимо сохранить отчет.
        :return:
        """
        full_cf_file_path = os.path.abspath(cf_file_path)
        full_report_path = os.path.abspath(report_path)
        logger.info(
            f'Сравниваю конфигурацию по соединению {self.connection} с файлом {full_cf_file_path} '
            f'для сохранения отчета по пути {full_report_path} ')
        params = [
            '/CompareCfg',
            '-FirstConfigurationType',
            'MainConfiguration',
            '-SecondConfigurationType',
            'File',
            '-SecondFile',
            f'{full_cf_file_path}',
            '-IncludeChangedObjects',
            '-IncludeDeletedObjects',
            '-IncludeAddedObjects',
            '–ReportType',
            'txt',
            '-ReportFormat',
            'Full',
            '-ReportFile',
            f'{full_report_path}'
        ]
        self.__execute_command(f'DESIGNER', params)
